<?php
/*********************************************************************************************************************************

	Function: sc_product()

	Purpose: to return HTML to display a single product

	Paramters:
		$prod_rn	(integer) 	record number of product
		$tpl		(string)	name of product template file

	History:
		File added on 09/27/2008 - for v3.0.0
		File modified on 08/05/2009 - for XSS vulnerability with product options

**********************************************************************************************************************************/
function sc_product($prod_rn, $tpl='product_thumbnail') {
	global $SC, $_GET;

	// need to know where item is in cart if editing it
	if (is_numeric($_GET['edit_item'])) {
		if (empty($SC['order']['items'])) {
			unset($_GET['edit_item']);
		} else {
			$hash_keys 	= array_keys($SC['order']['items']);
			$item_hash 	= $hash_keys[$_GET['edit_item']];
		}
	}
							
	
	// if $tpl isn't a full path, make it so
	if (!strstr($tpl,'/')) $tpl = sc_tpl($tpl);
	
	// if file doesn't exist, cya l8r allig8r
	if (!file_exists($tpl)) return false;

	// missing options fields are cached here by add_to_cart()...this sets them locally, and wipes the global var.
	$missing_opts = $SC['missing_options'];
	unset($SC['missing_options']);
	
	// set variable for use in template...these are the error messages from add to cart
	if (!empty($SC['add_cart_errors'])) {
		$Error_Message = implode('<br/>',$SC['add_cart_errors']);
		unset($SC['add_cart_errors']);
	}
	
	// set category rn....this is the category the customer is "in"
	if (is_numeric($_GET['crn'])) {
		$crn = $_GET['crn'];
	} else {
		$crn = sc_query('SELECT Categories_rn FROM REL_Products__Categories WHERE Products_rn = "'.$prod_rn.'" ORDER BY Position ASC LIMIT 1');
	}
	
	// store category as the last one viewed IF we are viewing a product detail
	if (is_numeric($_GET['rn']) && $_GET['action'] == 'show_detail') $SC['last_category_viewed'] = $crn;
	
	// set primary category for this product...this is the category that is at the top of the "Categories" field
	// on the product's record, and is the one that will be used to set various elements for this product,
	// including product options
	$primary_crn = sc_query("
		SELECT 
			Categories_rn 
		FROM 
			REL_Products__Categories
		WHERE
			Products_rn = '$prod_rn'
		ORDER BY
			Position ASC
		LIMIT 0,1
	");
	
	// grab an associative array of all data from Products table for this product
	$product = get_records("Products","*","record_number = '$prod_rn'",0,0,0,0,0,MYSQL_ASSOC,0);

	// workaround for warehouse feature change
	if (is_array($product)) $product['Number_in_Stock'] = get_sum("REL_Products__Warehouses","Stock","Products_rn = $product[record_number]");

	// if product doesn't exist, then include the product missing error template and exit out
	if (
			!$product 
		|| 	$product['Not_For_Sale']
		||	($product['Number_in_Stock'] <= 0 && $SC['out_of_stock_behavior'] == 1 && $SC['inventory_control'] == 1 && !isset($_GET['edit_item']))
		||	(strlen($_GET['edit_item']) && !is_numeric($_GET['edit_item']))	// if someone tried to change the edit_item variable
		) {
		if ($tpl == sc_tpl('product_detail')) {
			ob_start();
			include sc_tpl('error_product_missing');
			$error_html = ob_get_clean();
		} else {
			$error_html = '';
		}
		return $error_html;
	}
	
	if ($product['Not_For_Sale']) return false;
	
	// replace img dir constant with img dir URL
	$product['Description']	= str_replace("SC_IMG_DIR_DYN",SC_IMG_DIR_DYN,$product['Description']);
	
	// replace dyn_root constant with URL
	$product['Description']	= str_replace("SC_STORE_DIR_DYN",SC_STORE_DIR_DYN,$product['Description']);
	
	// this replaces PROD_RN with actual record number...useful for lightbox or anything else that needs a unique number
	$product['Description']	= str_replace("PROD_RN",$prod_rn,$product['Description']);
	
	// set a variable with the same name as each column name in the Products table, attempting to make each value
	// safe for XML
	extract(sc_xml_safe($product));
	
	$Name 			= sc_htmlspecialchars(strip_tags($Name));
	$Display_Name 	= $Display_Name ? $Display_Name : $Name;
			
	// link to show product in more detail
	$URL 			= get_product_url($prod_rn,$crn);
	
	
	// ******************** handle Product Options *******************************************************************

	// check the template to make sure it is set to show options...if not, no sense in doing all this
	if ($tpl) {
		foreach(file($tpl) as $tpl_line) $tpl_txt .= $tpl_line;
		if (strstr($tpl_txt, '$Options')) $show_options = true;
	} else {
		$show_options = true;
	}
	//$show_options = true;
	if ($show_options) {
		// clear cached options for this prod. record
		unset($SC['option_cache'][$prod_rn]);
		
		// set some variables
		$Required_Indicator = $SC['settings']['Required_Field_Indicator'];
		$Missing_Indicator 	= $SC['settings']['Missing_Field_Indicator'] == $Required_Indicator ? "" : $SC['settings']['Missing_Field_Indicator'];
		
		
		// grab options defined on this product's record
		$prod_opts = sc_query("
			SELECT
				o.*
				,po.record_number AS REL_rn
				,po.Options_rn
				,po.Position
			  	,po.Custom_Choices
				,'Products' AS Derived_Table
			FROM
				Options o
			INNER JOIN
				REL_Products__Options po
			ON
				o.record_number = po.Options_rn
			WHERE
				po.Products_rn = '$prod_rn'
			ORDER BY
				po.Position ASC
		", MYSQL_ASSOC, 1, 0, "Options_rn");

		// grab options defined on this product's primary category record
		if ($product['Merge_Options']) $cat_opts = sc_query("
			SELECT
				o.*
				,co.record_number AS REL_rn
				,co.Options_rn
				,co.Position
			  	,co.Custom_Choices
				,'Categories' AS Derived_Table
			FROM
				Options o
			INNER JOIN
				REL_Categories__Options co
			ON
				o.record_number = co.Options_rn
			WHERE
				co.Categories_rn = '$primary_crn'
			ORDER BY
				co.Position ASC
		", MYSQL_ASSOC, 1, 0, "Options_rn");
		
		// we need at least one of these arrays to be set to continue, otherwise there are no options
		if (is_array($prod_opts) || is_array($cat_opts)) {

			// Note: Unfortunately, the options from the product AND its primary category couldn't be
			// obtained in a single query, so this nonsense below is to work around that fact
			// 
			// Here is the gist of the logic below:
			// 
			// The keys of the 2 arrays above are the record numbers of the options from the Options table. 
			// We need to create an array containing all the (unique) keys from both arrays. Then, we will loop through them.
			// If the option was specified on the product record, we will show it based on those records ELSE
			// we will see if it was specified on the category record, and go that route instead. By doing this, the product will win out.
			// If the same option is specified for both the product AND it's primary category, the configuration of that option on the 
			// products record will be usd, and the one on the category record will be ignored.
			//
			// THE END :)

			// if options are on both product & category, and prod. is set to merge with category options
			if (is_array($prod_opts) && is_array($cat_opts) && $product['Merge_Options']) {
				$option_rns = array_unique(array_merge(array_keys($prod_opts), array_keys($cat_opts)));

				// if any options were duplicated on both the category AND the product, the array_unique will remove them, making the keys for the
				// $option_rns array out of order (example: 0, 1, 3, 4)
				// this will then mess up the for loop below...so, this fixes the keys
				$fixed_option_rns = array();
				foreach($option_rns as $option_rn) $fixed_option_rns[] = $option_rn;
				$option_rns = $fixed_option_rns;
								
			} elseif (is_array($prod_opts)) {
				$option_rns = array_keys($prod_opts);
			} elseif (is_array($cat_opts)) {
				$option_rns = array_keys($cat_opts);
			}
			
			// loop through each option record number
			for($x=0; $option_rns[$x]; $x++) {
				$opt_rn = $option_rns[$x];
				$opt = $prod_opts[$opt_rn] or $opt = $cat_opts[$opt_rn];
				
				// make option info safe for XHTML
				$opt = sc_xml_safe($opt);
				
				if ($opt['Required']) $required_present = true; // tells code farther down that at least 1 field is required

				unset($choices,$new_choices); // cleanup
				
				// grab choices for this option
				if ($opt['Custom_Choices']) {
					
					// portion of where clause to exclude out of stock choices
					// ONLY if out of stock behavior supports it AND custom_choices is set to 1
					//
					// remarked out on 2/19/2007 - for bug fix...out of stock items are no longer excluded from query
					//$out_of_stock_where = $SC['inventory_control'] && $opt['Use_Inventory_Control'] && ($SC['settings']['Out_of_Stock_Behavior'] == 1 || $SC['settings']['Out_of_Stock_Behavior'] == 3)  ? " AND cc.Stock >= 1" : "";

					// text/textarea/checkbox fields have only one choice, which is derived from the option itself....
					if ($opt['Type'] == "text" || $opt['Type'] == "textarea" || $opt['Type'] == "checkbox" || $opt['Type'] == 'file') {
						$choices = sc_query("
							SELECT
								 cc.record_number
								,cc.record_number AS Custom_Choices_rn
								,'$opt[Name]' AS Name
								,'$opt[Description]' AS Description
								,cc.Price
								,cc.Price_Operator
								,cc.Weight
								,cc.Weight_Operator
								,cc.Stock
								,'Custom_Choices' AS Source_Table
							FROM 
								Choices_Custom cc
							WHERE
								cc.REL_rn = '$opt[REL_rn]'
								AND cc.Source_Table = '$opt[Derived_Table]'
								#$out_of_stock_where
						", MYSQL_ASSOC,1);
					} else {
						$choices = sc_query("
							SELECT
								 cc.record_number
								,c.record_number AS Choices_rn
								,cc.record_number AS Custom_Choices_rn
								,c.Name
								,c.Description
								,c.Image
								,cc.Price
								,cc.Price_Operator
								,cc.Weight
								,cc.Weight_Operator
								,cc.Stock
								,'Custom_Choices' AS Source_Table
							FROM 
								Choices_Custom cc
							INNER JOIN
								Choices c
							ON
								cc.Choices_rn = c.record_number
							WHERE
								cc.REL_rn = '$opt[REL_rn]'
								AND cc.Source_Table = '$opt[Derived_Table]'
								#$out_of_stock_where
							ORDER BY
								cc.Position ASC
						", MYSQL_ASSOC,1);
					}
				} else {
					// text/textarea/checkbox fields have only one choice, which is derived from the option itself....
					if ($opt['Type'] == 'text' || $opt['Type'] == 'textarea' || $opt['Type'] == 'checkbox' || $opt['Type'] == 'file') {
						$choices[0]['record_number'] 	= $opt['record_number'];
						$choices[0]['Name'] 			= $opt['Name'];
						$choices[0]['Description'] 		= $opt['Description'];
						$choices[0]['Price'] 			= $opt['Default_Price'];
						$choices[0]['Price_Operator'] 	= $opt['Default_Price_Operator'];
						$choices[0]['Weight'] 			= $opt['Default_Weight'];
						$choices[0]['Weight_Operator'] 	= $opt['Default_Weight_Operator'];
						$choices[0]['Source_Table'] 	= "Options";
					} else { 
					
						$choices = sc_query("
							SELECT
								 c.record_number
								,c.record_number AS Choices_rn
								,c.Name
								,c.Description
								,c.Image
								,c.Default_Price AS Price
								,c.Default_Price_Operator AS Price_Operator
								,c.Default_Weight AS Weight
								,c.Default_Weight_Operator AS Weight_Operator
								,'Choices' AS Source_Table
							FROM
								Choices c
							INNER JOIN
								REL_Options__Choices oc
							ON
								c.record_number = oc.Choices_rn
							WHERE
								oc.Options_rn = '$opt_rn'
							ORDER BY
								oc.Position ASC
						", MYSQL_ASSOC, 1);
					}
				} // done setting $choices array
				
				// only continue w. this option if choices where found
				if (is_array($choices)) {
					// make info safe for XHTML
					$choices = sc_xml_safe($choices);
					
					// --------- loop through choices...set a variable named $Option_Field equal to the field for this option,
					// via output buffering
					unset($Option_Field); // cleanup 
					
					//$Option_Field_Name = "options[$x]";
					$Option_Field_Name = "option_$x";
					
					// the Position field in $opt is going to be set to what the Position column was in the DB
					// because we are pulling options from Products AND Categories, and possibly discarding some,
					// that position is not valid any longer....this makes the position match to the actual position the option
					// will have on the product page
					$opt['Position'] = $x;

					// select input has an option to add a blank choice at the top...this does that
					if ($opt['Type'] == "select" && $opt['Add_Blank_Choice']) {
						if (is_array($choices)) {
							$blank_choice['record_number'] = "BLANK_CHOICE";
							$new_choices[] = $blank_choice;
							foreach($choices as $new_choice) $new_choices[] = $new_choice;
							$choices = $new_choices;
						}
					}

					// store options and choices in cache....add_to_cart() uses this info to decide how to handle options
					$opt['choices'] = $choices;
					$SC['option_cache'][$prod_rn][] = $opt;

					// When we exclude choices that are out of stock, they still need to be in the loop below,
					// and they get skipped over numerically
					// In order to do that, we need to know when to show the footer, based on the last possible choice we can show...
					// This section of code will determine that via a $last_choice variable, which will be set to the index of the last choice in $choices
					// This only needs to be done when inventory control is on and we are hiding out of stock items
					
					if (
							$opt['Custom_Choices']																				// customer can only designate stock if they used custom choices
						&&	$SC['inventory_control']																			// inventory control is on in the store
						&&	$opt['Use_Inventory_Control']																		// inventory control is on for this option
						&&	($SC['settings']['Out_of_Stock_Behavior'] == 1 || $SC['settings']['Out_of_Stock_Behavior'] == 3)	// settings dont allow us to sell out of stock items
					) {
						$hiding_stock = 1;
						for($i=0; $choices[$i]; $i++) {
							if ($choices[$i]['Stock'] >= 1) $last_choice = $i;
						}
					} else {
						$hiding_stock = 0;
						$last_choice = count($choices) - 1;
					}
					
					$shown_header = 0;

					for($i=0; $choices[$i]; $i++) {
						
						//$show_header = $i === 0 ? true : false;
						//$show_footer = !$choices[$i+1] ? true : false;
						$choice = $choices[$i];
						
						// if choice is out of stock and the choice is not blank, and settings wont allow sale, skip it
						if (!$choice['Stock'] && $choice['record_number'] != 'BLANK_CHOICE' && $hiding_stock) continue;		
						
						// this will show the header on the first pass only
						$show_header	= !$shown_header ? 1 : 0;
						$shown_header	= 1;

						// show footer if we are on the last choice we can show
						$show_footer = $last_choice == $i ? 1 : 0;

						if ($choice['Price_Operator'] == "plus" && $choice['Price'] > 0) {
							$Price_Operator = ": + ";
						} elseif($choice['Price_Operator'] == "minus" && $choice['Price'] > 0) {
							$Price_Operator = ": - ";
						} else {
							$Price_Operator = "";
						}
						
						$Choice_Value			= $opt['Type'] == "select" || $opt['Type'] == "radio" || $opt['Type'] == "checkbox" ? $i : "";
						$Choice_Price 			= $choice['Price'] > 0 									? $SC['currency'].$choice['Price'] 		: "";
						$Choice_Text 			= $choice['Name'].$Price_Operator.$Choice_Price;
						$Choice_Width			= $opt['Width'];
						$Choice_Height			= $opt['Height'];
						$Choice_Break			= $opt['Type'] == "radio" && $opt['Break_After_Choice'] ? "<br />"								: "";
						$Option_Price			= $opt['Type'] == "text" || $opt['Type'] == "textarea" || $opt['Type'] == "checkbox" || $opt['Type'] == 'file' ? $Choice_Price 						: "";
						$Option_Price_Operator 	= $opt['Type'] == "text" || $opt['Type'] == "textarea" || $opt['Type'] == "checkbox" || $opt['Type'] == 'file' ? str_replace(": ","",$Price_Operator) 	: "";
						$Left_Paren				= $Option_Price ? "(" : "";
						$Right_Paren			= $Option_Price ? ")" : "";
						$Option_Type			= $opt['Type'];
						$Price_Attribute		= $choice['Price_Operator'] == 'minus' ? '-'.$choice['Price'] : $choice['Price'];
						
						// add "/ character" to end of price displayed as needed
						// and set a price point for QuickTotal to know how to charge for options
						if (
								!empty($Option_Price)
							&& 	($opt['Type'] == 'text' || $opt['Type'] == 'textarea')
							&&	$opt['Charge_by_Character']
							) {
							$Option_Price = $Option_Price .= ' / character';

							$Price_Point = $opt['Charge_for_Spaces'] ? 'char_spaces' : 'char';
						} else {
							// normal....charge for each option
							$Price_Point = 'auto';
						}
						
						// this handles setting the values of the options, when the item is being edited AFTER being added to the cart
						if (is_numeric($_GET['edit_item'])) {
							$compare_key 			= "option_$x-choice_$i";

							if ($opt['Type'] == "radio" || $opt['Type'] == "select" || $opt['Type'] == "checkbox") {
								$selected_text = $opt['Type'] == "select" ? 'selected="selected"' : 'checked="checked"';
								$Choice_Selected = is_array($SC['order']['items'][$item_hash]['options'][$compare_key]) ? $selected_text : "";
							} else {
								$compare_keys = array_keys($SC['order']['items'][$item_hash]['options']);
								foreach($compare_keys as $k) {
									if (strpos($k, "option_$x-") === 0) {
										$compare_key = $k;
										break;
									}
								}
								if ($compare_key) {
									// value is either text, or a filename
									$Choice_Value = $SC['order']['items'][$item_hash]['options'][$compare_key]['choice']['file'] ? $SC['order']['items'][$item_hash]['options'][$compare_key]['choice']['file'] : sc_htmlspecialchars($SC['order']['items'][$item_hash]['options'][$compare_key]['choice']['text']);
								}
							}
							
						} elseif ($_GET['add_to_cart']) {
						
							// this handles setting the values of the options if the record was just submitted for add to cart
							// and it failed validation
						
							if ($opt['Type'] == "radio" || $opt['Type'] == "select") {
								$selected_text = $opt['Type'] == "select" ? "selected" : "checked";
								$Choice_Selected = is_numeric($_GET["option_$x"]) && $_GET["option_$x"] == $i ? $selected_text : "";
							} elseif ($opt['Type'] == "checkbox") {
								$Choice_Selected = $_GET["option_$x"] == '0' ? "checked" : "";
							} elseif ($opt['Type'] == "text" || $opt['Type'] == "textarea") {
								$Choice_Value = get_magic_quotes_gpc() ? stripslashes($_GET["option_$x"]) : $_GET["option_$x"];
								$Choice_Value = sc_htmlspecialchars($Choice_Value);
							} elseif ($opt['Type'] == 'file') {
								
							}
						}
						
						// handle field indicators - required, missing, etc...
						unset($Field_Indicator, $Error_Class);
						// decide whether to show required field indicator OR missing indicator
						if ($missing_opts[$x]) {
							$Field_Indicator = $SC['settings']['Missing_Field_Indicator'];
							if ($SC['settings']['Missing_Field_Colored']) {
								$Error_Class = $opt['Type'] == "radio" ? "error_field_radios" : "error_field";
							} else {
								unset($Error_Class);
							}
							$missing_present = true; // tells code farther down that at least 1 field is missing
						} elseif ($opt['Required']) {
							$Field_Indicator = $SC['settings']['Required_Field_Indicator'];
							$required_present = true; // tells code farther down that at least 1 field is required
						}
						
						// use output buffering to grab result of the include, so we can store it in a var and use it later
						ob_start(); 

						if ($opt['Type'] == "select") {
							include sc_tpl('option_select');
						
						} elseif ($opt['Type'] == "radio") {
	
							include sc_tpl('option_radio');
						
						} elseif ($opt['Type'] == "checkbox") {
	
							include sc_tpl('option_checkbox');
						
						} elseif ($opt['Type'] == "text") {
							include sc_tpl('option_text');
						
						} elseif ($opt['Type'] == "textarea") {
							
							include sc_tpl('option_textarea');

						} elseif ($opt['Type'] == 'file') {
							$Allowed_Size = $opt['File_Size'];
							
							// grab an array of all allowed file extensions from File_Types table
							$allowed_extensions = sc_query('SELECT `Extension` FROM `File_Types` WHERE record_number IN('.str_replace('^^',',',$opt['File_Types']).')',MYSQL_ASSOC,1,0,'',1);
							if (is_array($allowed_extensions)) {
								$Allowed_File_Types = implode(', ',$allowed_extensions);
							}
							
							// set variables to class names to control initial visibility of edit and saved field sections
							$Saved_Class	= is_numeric($_GET['edit_item']) ? 'sc_show'	: 'sc_hide';
							$Edit_Class		= is_numeric($_GET['edit_item']) ? 'sc_hide'	: 'sc_show';
							
							
							include sc_tpl('option_file');
							
							$file_upload_on = true;
						}
						
						$Option_Field .= ob_get_clean(); 

						
					}
					// --------- END OF loop through choices...done setting up $Option_Field var ---------------------------------------
					
					
					
					
					
					// replace img dir constant with actual URL to img dir
					$opt['Instruction']	= str_replace("SC_IMG_DIR_DYN",SC_IMG_DIR_DYN,$opt['Instruction']);
					
					// use output buffering to grab result of the include, so we can store it in a var named $Options and use it later
					// setup vars named after the columns in the Options table...prefix with "Option" to make them unique
					foreach(array_keys($opt) as $key) {
						$var_name 	= "Option_$key";
						$$var_name 	= $opt[$key];
					}
					
					// if intruction is blank, use option name
					if (!$opt['Instruction']) $Option_Instruction = $Option_Name;

					// below handles stuff to show the option detail popup page
					unset($Zoom_In_Image, $Custom_Detail_Link, $Detail_Link_Right, $Detail_Link_Left, $Extra_Br); // cleanup
					
					if ($opt['Show_Detail_Page']) {
						// the url for the detail page link...supports a custom URL
						if ($opt['Detail_Page_Content'] == "autogenerate") {
							$option_detail_url = SC_CP_DIR_DYN."/option_detail.php?REL_rn=$opt[REL_rn]&amp;t=".strtolower($opt['Derived_Table'][0]).SID_URL;
						} else {
							$option_detail_url = $opt['Detail_Page_URL'].str_replace('&amp;','?',SID_URL);
						}
						
						if ($opt['Detail_Page_Size'] == "custom") {
							$option_detail_href = "javascript: var optDetail = window.open('$option_detail_url', 'optDetail', 'width=$opt[Detail_Page_Width], height=$opt[Detail_Page_Height], scrollbars=1, resizable=1')";
							$target = "";
						} else {
							$option_detail_href = $option_detail_url;
							$target = "target=\"_blank\"";
						}
						
						$detail_links = explode("^^", $opt['Detail_Links']); // turn this into an array
						if (in_array("instruction", $detail_links)) {
							$Option_Instruction = "<a href=\"$option_detail_href\" $target>$Option_Instruction</a>";
						}
	
						if (in_array("image", $detail_links)) {
							$Zoom_In_Image = " <a href=\"$option_detail_href\" $target>".sc_img("zoom_in",'tag')."</a>";
						}
	
						if (in_array("custom", $detail_links)) {
							$Custom_Detail_Link = " <a class=\"custom_detail_href\" href=\"$option_detail_href\" $target>$opt[Custom_Link_Text]</a>";
						}
						
						// control placement of custom detail link and image detail link
						if ($Zoom_In_Image || $Custom_Detail_Link) {
							// if placement wasn't chosen, set defaults
							if (!$opt['Detail_Link_Placement']) {
	
								switch($opt['Type']) {
									case "text":
								    case "select":
								    case "checkbox":
								    	$opt['Detail_Link_Placement'] = "right";
								    	break;
								    case "textarea":
								    case "radio":
								    	$opt['Detail_Link_Placement'] = "leftbr";
								    	break;
								    default:
								    	$opt['Detail_Link_Placement'] = "right";
								}							
							}
							
							// set fields for options.php templates, based on link placement above
							$Detail_Link_Left 	= $opt['Detail_Link_Placement'] == "left" || $opt['Detail_Link_Placement'] == "leftbr" ? true : false;
							if ($opt['Detail_Link_Placement'] == "leftbr") {
								$Detail_Br_Left_Open = "<div>";
								$Detail_Br_Left_Closed = "</div>";
							}
							$Detail_Link_Right 	= $opt['Detail_Link_Placement'] == "right" || $opt['Detail_Link_Placement'] == "rightbr" ? true : false;
							if ($opt['Detail_Link_Placement'] == "rightbr") {
								$Detail_Br_Right_Open = "<div>";
								$Detail_Br_Right_Closed = "</div>";
							}
						}
					
					} else {
						// cleanup
						unset($Extra_Br, $Zoom_In_Image, $Custom_Detail_Link, $Detail_Link_Left, $Detail_Br_Right_Open, $Detail_Br_Right_Close, $Detail_Br_Left_Open, $Detail_Br_Left_Close, $Detail_Br_Right);
						// surround instruction with span so it vertically aligns middle
						$Option_Instruction = "<span>$Option_Instruction</span>";
					}
	
					// for certain fields, if there is no link below it, we need to add a br, to make spacing correct
					if (!$Detail_Link_Right && ($opt['Type'] == "textarea" || ($opt['Type'] == "radio" && $opt['Break_After_Choice']))) {
						$Extra_Br = "<div class=\"extra_br\"></div>";
					}
					
					
					
					// next 2 lines control whether or not the table tag will show in the resulting code
					$show_header = $x === 0 			? true : false;
					$show_footer = !$option_rns[$x+1]	? true : false;
					ob_start(); 
					include sc_tpl('options');
					$Options .= ob_get_clean();

					// ---------- END OF setting up $Options var -----------------------------------------------------------------------
				}
			} // end of loop through options
		}
	}
	
	
	// ******************** END OF handling Product Options *******************************************************************
	
	
	
	
	
	
	
	
	
	/************************************************************************************
		Handle Sales Agreements	
	************************************************************************************/
	$agreements = sc_query("
		SELECT
			`sa`.*
		FROM
			`Sales_Agreement` `sa`
		INNER JOIN
			`REL_Products__Sales_Agreement` `psa`
		ON
			`sa`.`record_number` = `psa`.`Sales_Agreement_rn`
		WHERE
			`psa`.`Products_rn` = '$prod_rn'
		ORDER BY
			`psa`.`Position` ASC
	", MYSQL_ASSOC, 1);
			
	$Agreements = array(); // for template use
	if (is_array($agreements)) {
		for($x=0; $agreements[$x]; $x++) {
			// make safe for XHTML
			$agreement = sc_xml_safe($agreements[$x]);
			
			// replace img dir constant with img dir URL
			$agreement['Agreement'] = str_replace("SC_IMG_DIR_DYN",SC_IMG_DIR_DYN,$agreement['Agreement']); 
			
			// instance for CSS
			if (!$agreements[$x+1]) {
				$agreement['Instance'] = 'last';
			} elseif ($x == 0) {
				$agreement['Instance'] = 'first';
			} else {
				$agreement['Instance'] = '';
			}
			
			// add display name if not present
			if (!$agreement['Display_Name']) $agreement['Display_Name'] = $agreement['Name'];
			
			// name is used in a field, needs to contain no HTML whatsoever
			$agreement['Name'] = sc_htmlspecialchars($agreement['Name']);

			$agreement['Print_URL'] = SC_CP_DIR_DYN."/sales_agreement.php?agreement_rn=$agreement[record_number]".SID_URL;

			$Agreements[] = $agreement;
		}
	}

	
	
	
	/******************************************************
		Setup vars and hidden form fields for template
	******************************************************/
	$Badge_Image			= '';	// default to no badge. this may change to "new" or "sale", or "10% off", etc....
	$Link_Title				= valid_non_html($Name) ? sc_htmlspecialchars($Name) : '';
	$Thumbnail_Image		= sc_img($product['Thumbnail_Image'], 'all','',$Link_Title);
	$Image					= sc_img($product['Image'], 'all','',$Link_Title);
	$Large_Image			= sc_img($product['Large_Image'], 'all','',$Link_Title);
	$Featured_Image			= sc_img($product['Featured_Image'], 'all','',$Link_Title);
	$Thumbnail_Image		= empty($Thumbnail_Image) ? sc_img($SC['settings']['Missing_Thumbnail_Image'], 'all','',$Link_Title) : $Thumbnail_Image;
	$Image					= empty($Image) ? sc_img($SC['settings']['Missing_Image'], 'all','',$Link_Title) : $Image;
	$Form_Action 			= SC_STORE_PAGE_HTTP;
	$Price_Label 			= $SC['settings']['Base_Price_Label'];
	$Add_to_Cart_Message	= !empty($SC['msg_add_to_cart']) ? $SC['msg_add_to_cart'] : '';
	$Quantity_Value			= is_numeric($_GET['edit_item']) && !empty($item_hash) ? $SC['order']['items'][$item_hash]['qty'] : 1;

	
	/******************************************************
		Setup tooltip if enabled
	******************************************************/
	if (
			$SC['settings']['Tooltip_Enabled']		// enabled for entire store 
		&& 	$SC['settings']['Tooltip_Products'] 	// and enabled for all products
		&& 	$product['Tooltip_Enabled']				// and enabled for THIS product
	) {
		// if using brief description field
		if ($product['Tooltip_Source'] == 1) {
			$Tooltip = $product['Brief_Description'];
		} else {
			// if using custom field
			$Tooltip = $product['Tooltip_Text'];
		}
		
		// if we have no tooltip at all, use default
		if (empty($Tooltip)) {
			$Tooltip = $SC['settings']['Tooltip_Products_Default'];
		} elseif ($SC['settings']['Tooltip_Products_Append']) {
			// if here, we have tooltip content and the store settings say we should also 
			// append the default tooltip to the end of it
			$Tooltip .= "<br/><br/>".$SC['settings']['Tooltip_Products_Default'];
		}
		
		// fix to allow html
		$Tooltip = sc_htmlspecialchars($Tooltip);
	} else {
		$Tooltip = '';
	}
	// ID to put in anchor that contains tip, so we can trigger it manually if needed
	$Tooltip_ID = 'tip_'.str_replace(' ','',microtime());
	
	
	// form ID...very important for buttons to work
	$Form_ID = md5(uniqid(rand(),true));
	
	// this is used to report back to calling functions, to automatically align boxes to baseline
	$GLOBALS['thumb_height'] = $Thumbnail_Image['height'];
	
	$hidden_fields = array(
		'add_to_cart'				=> 1,
		'discount_price' 			=> '',					// this is for quick total
		'discounted_qty' 			=> '',					// this is for quick total
		'discounted_qty_in_cart' 	=> '',					// this is for quick total
		'prod_rn'					=> $prod_rn,
		'microtime'					=> microtime(),
		'edit_item'					=> is_numeric($_GET['edit_item']) ? $_GET['edit_item'] : '',
	);
	
	// handle form tag vars for use with file uploads
	if ($file_upload_on) {
		$Form_Enctype	 = 'enctype="multipart/form-data"';
		$Form_Method	 = 'post';
		$hidden_fields['file_option_on'] = 1;
	} else {
		$Form_Method	= 'get';
		$Form_Enctype	= '';
	}
	

	// For prod calc module, grab contents of template that controls the appearance of the total
	if (isset($SC['modules']['quick_total'])) {
		$quick_total_tpl 	= sc_tpl('quick_total');
		$Small_Check_Image 	= sc_img('small_check','all','quick_total');
		
		if (is_file($quick_total_tpl)) {
			ob_start();
			include $quick_total_tpl;
			$Quick_Total = ob_get_clean();
		}
	}
	
	// For review module
	if ($SC['modules']['review']['settings']['Enabled']) {
		// no sense in getting reviews if they aren't going to show them. Check for the $Reviews variable in the template before calling get_reviews
		foreach(file($tpl) as $tpl_line) $tpl_txt .= $tpl_line;
		if (strstr($tpl_txt, '$Reviews')) $show_reviews = true;
		$review_mode 	= ($tpl == sc_tpl('product_thumbnail') || $tpl == sc_tpl('new_product_preview_td')) ? 'prod thumb' : 'prod detail';
		$Reviews 		= $show_reviews ? get_reviews($review_mode,$product['record_number']) : '';
	}

	if(security_level("Store Admin")){
		$Admin_Link = "<a class=\"sc_smaller\" title=\"edit product\" href=\"".SC_CP_PAGE_DYN."?edit_records=1&amp;selected_record_number=$product[record_number]&amp;table=Products".SID_URL."\">[edit]</a>";
	} else {
		unset($Admin_Link);
	}
	
	/*********************************************************************
		Handle discounts for products
	*********************************************************************/
	
	// grab discount info
	$discount = get_discount('product',$prod_rn);
	// we are only doing this for show, so if it's a buy x get y, we can't show the sale price
	if ($discount['buy_x_get_y'] == 'true') unset($discount);
	
	if($product['Show_Base_Price']) {
		if ($discount) {
			$Regular_Price 			= $Base_Price;
			$Regular_Price_Label	= sc_xml_safe($SC['settings']['Regular_Price_Label']);
			$Base_Price 			= $discount['discount_price'];
			$Discount_Price 		= $discount['discount_price'];
			$Savings_Amount 		= $discount['savings_amt'];
			$Savings_Percent 		= $discount['savings_percent'];
		}
		
		$hidden_fields['b_price'] = $Base_Price;
		$Base_Price = sc_money_format($Base_Price,1);
		
	} else {

		$hidden_fields['b_price'] = $Base_Price;
		unset($Base_Price);
	}

	$Add_to_Cart_Class 	= is_numeric($_GET['edit_item']) ? 'update_cart2_btn' : 'add_to_cart_btn';
	$Add_to_Cart_Text 	= is_numeric($_GET['edit_item']) ? 'Update Cart' : 'Add to Cart';
	
	$Hidden_Fields = SID_FLD;
	foreach($hidden_fields as $field => $value) $Hidden_Fields .= "<input type=\"hidden\" name=\"$field\" value=\"".sc_htmlspecialchars($value)."\" />\r\n";
	 
	// if inventory control is enabled, and item is out of stock
	$Allow_Purchase 		= 1;
	$Out_of_Stock_Message 	= '';
	// if product is out of stock
	if($SC['inventory_control'] && $product['Number_in_Stock'] < 1){

		// set the out of stock message IF it should be shown
		if ($SC['out_of_stock_behavior'] == 3 || $SC['out_of_stock_behavior'] == 4) {
			$Out_of_Stock_Message = $product['Out_of_Stock_Indicator'] ? $product['Out_of_Stock_Indicator'] : $SC['settings']['Out_of_Stock_Indicator'];
			$Out_of_Stock_Message = str_replace("SC_IMG_DIR_DYN",SC_IMG_DIR_DYN,$Out_of_Stock_Message);
		}
		
		// merchant doesn't want this purchased when out of stock
		if ($SC['out_of_stock_behavior'] == 3) $Allow_Purchase = 0;
	}
			


	// below checks to see if product is new based on "New" record in the Admin_Options table
	$today				= date("Y-m-d H:i:s"); 
	$today				= strtotime($today);
	$product_date		= $product["Date_Added_to_Cart"];
	$product_date		= strtotime($product_date);
	$date_difference	= (($today - $product_date)/86400); 
	$new_length 		= $SC['settings']['Advertise_New_Products'] ? $SC['settings']['New'] : 0;
	$date_difference 	= number_format($date_difference, 0,"","");			// strip decimal places to make age of product in days a whole number

	// set badge to "NEW" for new products (can get overriden by sale badgers below)
	$tpl_type 		= strstr($tpl,'detail') ? 'detail' : 'thumbnail';	// indicate which type of template we are dealing with
	$Badge_Image 	= !empty($new_length) && $new_length >= $date_difference ? sc_img('badge_new_'.$tpl_type) : '';
	
	// if one is set in the discount record...if so, use it, if not, use what $tpl is already set to
	$Discount_Class = $discount['template'] == 'discount' ? 'discounted' : '';

	// setup things that are only shown when the product is set to display "as discounted"
	if ($Discount_Class) {
		
		$Price_Label 		= !empty($discount['price_label']) ? $discount['price_label'] : $Price_Label;
		$Show_Regular_Price = $discount['Show_Regular_Price'];		

		// setup badges (we don't need no stinking badges) IF they are enabled
		if ($SC['settings']['Discount_Badges_Enabled']) {
			$discount['Show_Badge'] = $discount['Show_Badge'] == 'default' ? $SC['settings']['Discount_Show_Badge'] : $discount['Show_Badge'];
			$discount['Badge_Type'] = $discount['Show_Badge'] == 'default' ? $SC['settings']['Discount_Badge_Type'] : $discount['Badge_Type'];
			
			// if badges are allowed for this template
			if (
					($tpl_type == 'thumbnail' && $discount['Show_Badge'] == 'thumbnail')
				||	($tpl_type == 'detail' && $discount['Show_Badge'] == 'detail')
				||	($discount['Show_Badge'] == 'both')
			) {
				// setup discount badge names based on settings
				if ($discount['Badge_Type'] == 'sale') {
					$badge_name = "badge_sale_sale_$tpl_type";
				} else {
					// auto...
					// array containing allowed percentage
					$allowed_percents = array(5,10,15,20,25,30,33,35,40,45,50,55,60,65,70,75,80,85,90,95,100);
					
					// boolean to tell if we can use percent image or not
					$use_percent = in_array($discount['savings_percent'],$allowed_percents) ? 1 : 0;
					
					$badge_name	= $use_percent ? "badge_sale_$discount[savings_percent]_$tpl_type" 	: "badge_sale_sale_$tpl_type";
				}
	
				// try to grab designated badge
				$Badge_Image = sc_img($badge_name);
			}
		}
	}

	// grab HTML to display all possible discounts for this product and store in a var
	if ($tpl == sc_tpl('product_detail')) {
		$Discounts = show_discounts("product_detail",$prod_rn);
	}

	// add hidden field to allow JS for the prod. calc module to know if there are any discounts for this item
	if ($Discounts !== false) $Hidden_Fields .= '<input type="hidden" name="discounts_present" value="1" />';
	
	ob_start();
	include $tpl;
	$html = ob_get_clean();
	
	return $html;
}
?>